package com.example.cameraxtestappjava;

import static com.example.cameraxtestappjava.camera.SegpassCamera.REQUEST_CAMERA_PERMISSION;

import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.TextureView;
import android.widget.ImageButton;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import androidx.core.app.ActivityCompat;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;

import com.example.cameraxtestappjava.camera.SegpassPermissionListener;
import com.example.cameraxtestappjava.camera.SegpassCamera;
import com.example.cameraxtestappjava.camera.SegpassCameraCallback;
import com.example.cameraxtestappjava.databinding.ActivityMainBinding;

public class MainActivity extends AppCompatActivity implements SegpassPermissionListener {

    // binding
    ActivityMainBinding binding;

    // Camera2
    private static final String TAG = "MainActivity";
    SegpassCamera mSegpassCamera;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        setUpCamera2();

        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });
    }

    // Camera 2
    private void setUpCamera2() {

        // Initialize visual elements in host app
        TextureView mTextureView = binding.c2Camera.tvCameraTextureView;
        ImageButton mTakePictureButton = binding.c2Camera.btnTakepicture;

        // Initialized CustomCamera2 in host app
        //mCustomCamera2 = new CustomCamera2(this, mTextureView);
        mSegpassCamera = new SegpassCamera(this, mTextureView, this);
        mSegpassCamera.init(); // Do not skip this!!!

        mTakePictureButton.setOnClickListener(v -> mSegpassCamera.takePicture(new SegpassCameraCallback() {
            @Override
            public void onPictureTakenSuccess(String message) {
                Toast.makeText(MainActivity.this, message, Toast.LENGTH_LONG).show();
            }

            @Override
            public void onPictureTakenFailError(String error) {
                Toast.makeText(MainActivity.this, error, Toast.LENGTH_LONG).show();
            }
        }));
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == REQUEST_CAMERA_PERMISSION) {
            if (grantResults[0] == PackageManager.PERMISSION_DENIED) {
                // close the app
                Toast.makeText(MainActivity.this, "Sorry!!!, you can't use this app without granting permission", Toast.LENGTH_LONG).show();
                finish();
            } else {
                onPermissionGranted();
            }
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        mSegpassCamera.resumeCamera();
    }

    @Override
    protected void onPause() {
        mSegpassCamera.pauseCamera();
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        mSegpassCamera.destroyCamera();
        super.onDestroy();
    }

    @Override
    public void onPermissionGranted() {
        Toast.makeText(MainActivity.this, "You have permissions to use the camera!", Toast.LENGTH_LONG).show();
    }

    @Override
    public void onPermissionDenied() {
        ActivityCompat.requestPermissions(this,
                new String[]{android.Manifest.permission.CAMERA,
                        android.Manifest.permission.WRITE_EXTERNAL_STORAGE},
                REQUEST_CAMERA_PERMISSION
        );
    }
}